//
//  Passer_RatingAppDelegate.m
//  Passer Rating
//
//  Created by Xcode User on 6/19/11.
//  Copyright 2011 Fritz Anderson. All rights reserved.
//

#import "Passer_RatingAppDelegate.h"
#import "Passer.h"
#import "Game.h"
#import "PasserListController.h"

NSManagedObjectContext *		gMOC = nil;

NSString *	 const	kPrefStartupAction	= @"PRStartupAction";
NSString *	 const	kStartupKeepData	= @"PRStartupDontChange";
NSString *	 const	kStartupUseDemo		= @"PRStartupReplaceWithDemo";
NSString *	 const	kStartupRemoveDemo	= @"PRStartupRemoveDemo";
NSString *	 const	kStartupRemoveAll	= @"PRStartupRemoveAll";

NSString * const	kFinalSaveFailed	= @"PRFinalSaveFailed";

NSString * const	WT9TErrorDomain		= @"com.wt9t.err";

@implementation Passer_RatingAppDelegate

@synthesize window = _window;
@synthesize managedObjectContext = __managedObjectContext;
@synthesize managedObjectModel = __managedObjectModel;
@synthesize persistentStoreCoordinator = __persistentStoreCoordinator;
@synthesize navigationController = _navigationController;

+ (void) initialize
{
	NSUserDefaults *		defaults = [NSUserDefaults standardUserDefaults];
	[defaults registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
								 kStartupUseDemo, kPrefStartupAction,
								 [NSNumber numberWithBool: NO], kFinalSaveFailed,
								 nil]];
}

- (BOOL) application: (UIApplication *) application
didFinishLaunchingWithOptions: (NSDictionary *) launchOptions
{
	UIAlertView *			alert = nil;
    
    // Force loading of the managed object context  	
	if (! self.managedObjectContext) {
		alert = [[UIAlertView alloc] initWithTitle: @"Can't start up"
										   message: @"It was impossible to set up the application's"
                 " database. Either your device is out of storage,"
                 " or fritza@wt9t.com has blown it. Please let him"
                 " know about the problem. All you can do now is"
                 " press the Home button."
										  delegate: nil
								 cancelButtonTitle: @"OK"
								 otherButtonTitles: nil];
		[alert autorelease];
		[alert show];
        
		[_window makeKeyAndVisible];
		return NO;
	}
	
	NSUserDefaults *		defaults = [NSUserDefaults standardUserDefaults];
	BOOL					prevSaveFailed = [defaults boolForKey: kFinalSaveFailed];
	if (prevSaveFailed) {
		alert = [[UIAlertView alloc] initWithTitle: @"Could not save data"
										   message: @"The last time you exited this app, it tried to"
                 " save the last of your changes, and failed. You"
                 " may notice some data loss. Let fritza@wt9t.com"
                 " know about the problem."
										  delegate: nil
								 cancelButtonTitle: @"OK"
								 otherButtonTitles: nil];
		[alert autorelease];
	}
	[defaults setBool: NO forKey: kFinalSaveFailed];
	
#if 0
	NSString *				whatToDo = [defaults stringForKey: kPrefStartupAction];
	if (! whatToDo)
		whatToDo = kStartupKeepData;
#else
    NSString *              whatToDo = kStartupUseDemo;
#endif
	
	BOOL		shouldRemoveAllData = NO;
	BOOL		shouldRemoveDemoData = NO;
	BOOL		shouldAddDemoData = NO;
	
	if ([whatToDo isEqualToString: kStartupUseDemo]) {
		shouldRemoveAllData = YES;
		shouldAddDemoData = YES;
	}
	else if ([whatToDo isEqualToString: kStartupRemoveDemo]) {
		shouldRemoveDemoData = YES;
	}
	else if ([whatToDo isEqualToString: kStartupRemoveAll]) {
		shouldRemoveAllData = YES;
	}
	
	NSError *	error;
	BOOL		success = YES;
    
	if (shouldRemoveAllData) {
		NSArray *		passers = [Passer passersSortedBy: nil inContext: gMOC];
		for (Passer * passer in passers)
			[gMOC deleteObject: passer];
	}
	
	if (shouldAddDemoData) {
		NSString *	path = [[NSBundle mainBundle] pathForResource: @"sample-data"
                                                          ofType: @"csv"];
		success = [Game loadFromCSVFile: path intoContext: gMOC error: &error];
	}
	
	if (shouldRemoveDemoData) {
		NSString *	path = [[NSBundle mainBundle] pathForResource: @"sample-data"
                                                          ofType: @"csv"];
		success = [Passer removePassersInCSVFile: path forContext: gMOC error: &error];
	}
	
	if (! success) {
		NSLog(@"%s: Could not reinitialize", __PRETTY_FUNCTION__);
		NSLog(@"%s: Error = %@", __PRETTY_FUNCTION__, error);
		NSLog(@"%s: userDict = %@", __PRETTY_FUNCTION__, error.userInfo);
		
		[gMOC rollback];
		
		alert = [[UIAlertView alloc] initWithTitle: @"Internal problem reinitializing data"
										   message: @"You used the Settings app to ask that the passer"
                 " data be reinitialized. Due to an internal error,"
                 " this could not be done. The database is"
                 " unchanged. Please let fritza@wt9t.com know."
										  delegate: nil
								 cancelButtonTitle: @"OK"
								 otherButtonTitles: nil];
		[alert autorelease];
	}
	else if (shouldAddDemoData || shouldRemoveAllData || shouldRemoveDemoData) {
		if (! [gMOC save: &error]) {
			NSLog(@"%s: Could not save MOC after reinitializing data.", __PRETTY_FUNCTION__);
			NSLog(@"%s: Error = %@", __PRETTY_FUNCTION__, error);
			NSLog(@"%s: userDict = %@", __PRETTY_FUNCTION__, error.userInfo);
			
			//	Recovery:
			[gMOC rollback];
			
			alert = [[UIAlertView alloc] initWithTitle: @"Problem revising data"
											   message: @"You asked for modifications to the passer"
                     " database, but these could not be saved. The"
                     " database is unchanged. Please let"
                     " fritza@wt9t.com know."
											  delegate: nil
									 cancelButtonTitle: @"OK"
									 otherButtonTitles: nil];
			[alert autorelease];
		}
	}
	
	[defaults setObject: kStartupKeepData forKey: kPrefStartupAction];
	[defaults synchronize];
    
	[alert show];		
    
    [_window addSubview: [_navigationController view]];
    [_window makeKeyAndVisible];
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    /*
     Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
     Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
     */
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    /*
     Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
     */
}

- (void)applicationWillTerminate:(UIApplication *)application
{
	NSUserDefaults *		defaults = [NSUserDefaults standardUserDefaults];
    //  The save failed unless proved otherwise:
    [defaults setBool: YES forKey: kFinalSaveFailed];

    NSError *				error;
    if (self.managedObjectContext != nil) {
        if ([self.managedObjectContext hasChanges] && ! [self.managedObjectContext save: &error])
			NSLog(@"%s: save error %@, %@", __PRETTY_FUNCTION__, error, [error userInfo]);
		else 
			[defaults setBool: NO forKey: kFinalSaveFailed];
            //  It was proved otherwise
    }
	[defaults synchronize];
}

- (void) dealloc
{
    [_window release];
    [__managedObjectContext release];
    [__managedObjectModel release];
    [__persistentStoreCoordinator release];
    [_navigationController release];
    [super dealloc];
}

- (void)awakeFromNib
{
    PasserListController *rootViewController = (PasserListController *)[self.navigationController topViewController];
    rootViewController.managedObjectContext = self.managedObjectContext;
}

#pragma mark - Core Data stack

/**
 Returns the managed object context for the application.
 If the context doesn't already exist, it is created and bound to the persistent store coordinator for the application.
 */
- (NSManagedObjectContext *)managedObjectContext
{
    if (__managedObjectContext != nil)
        return __managedObjectContext;
    
    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (coordinator != nil)
    {
        __managedObjectContext = [[NSManagedObjectContext alloc] init];
        [__managedObjectContext setPersistentStoreCoordinator:coordinator];
        gMOC = __managedObjectContext;
   }
    return __managedObjectContext;
}

/**
 Returns the managed object model for the application.
 If the model doesn't already exist, it is created from the application's model.
 */
- (NSManagedObjectModel *)managedObjectModel
{
    if (__managedObjectModel != nil)
        return __managedObjectModel;
    NSURL *modelURL = [[NSBundle mainBundle] URLForResource:@"Passer_Rating" withExtension:@"momd"];
    __managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];    
    return __managedObjectModel;
}

/**
 Returns the persistent store coordinator for the application.
 If the coordinator doesn't already exist, it is created and the application's store added to it.
 */
- (NSPersistentStoreCoordinator *)persistentStoreCoordinator
{
    if (__persistentStoreCoordinator != nil)
    {
        return __persistentStoreCoordinator;
    }
    
    NSURL *storeURL = [[self applicationDocumentsDirectory] URLByAppendingPathComponent:@"Passer_Rating.sqlite"];
    
    NSError *error = nil;
    __persistentStoreCoordinator = 
            [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:[self managedObjectModel]];
    if (![__persistentStoreCoordinator addPersistentStoreWithType: NSSQLiteStoreType
                                                    configuration: nil
                                                              URL: storeURL
                                                          options: nil
                                                            error: &error]) {
        /*
         Replace this implementation with code to handle the error appropriately.
         
         abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. If it is not possible to recover from the error, display an alert panel that instructs the user to quit the application by pressing the Home button.
         
         Typical reasons for an error here include:
         * The persistent store is not accessible;
         * The schema for the persistent store is incompatible with current managed object model.
         Check the error message to determine what the actual problem was.
         
         
         If the persistent store is not accessible, there is typically something wrong with the file path. Often, a file URL is pointing into the application's resources directory instead of a writeable directory.
         
         If you encounter schema incompatibility errors during development, you can reduce their frequency by:
         * Simply deleting the existing store:
         [[NSFileManager defaultManager] removeItemAtURL:storeURL error:nil]
         
         * Performing automatic lightweight migration by passing the following dictionary as the options parameter: 
         [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES], NSMigratePersistentStoresAutomaticallyOption, [NSNumber numberWithBool:YES], NSInferMappingModelAutomaticallyOption, nil];
         
         Lightweight migration will only work for a limited set of schema changes; consult "Core Data Model Versioning and Data Migration Programming Guide" for details.
         
         */
        NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
        abort();
    }    
    
    return __persistentStoreCoordinator;
}

#pragma mark - Application's Documents directory

/**
 Returns the URL to the application's Documents directory.
 */
- (NSURL *)applicationDocumentsDirectory
{
    return [[[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask] lastObject];
}

@end
