//
//  rating.c
//  PasserRatingTool
//
//  Created by Xcode User on 7/1/10.
//  Copyright 2010 Fritz Anderson. All rights reserved.
//

#include "rating.h"

static
double pinPassingComp(double component)
{
	if (component < 0.0)
		return 0.0;
	else if (component > 2.375)
		return 2.375;
	else
		return component;
}

float
passer_rating(int comps, int atts, int yds, int tds, int ints)
{
	//	See http://en.wikipedia.org/wiki/Quarterback_Rating
    
    if (atts == 0)
        return 0.0;
    
	double		completionComp =
                    (((double) comps / atts) * 100.0 - 30.0) / 20.0;
	completionComp = pinPassingComp(completionComp);
	
	double		yardageComp =
                    (((double) yds / atts) - 3.0) / 4.0;
	yardageComp = pinPassingComp(yardageComp);
	
	double		touchdownComp =
                    20.0 * (double) tds / atts;
	touchdownComp = pinPassingComp(touchdownComp);
	
	double		pickComp = 
                    2.375 - (25.0 * (double) ints / atts);
	pickComp = pinPassingComp(pickComp);
	
	double retval =  100.0 * (completionComp +
							  yardageComp +
							  touchdownComp +
							  pickComp) / 6.0;
	return retval;	
}
