//
//  Passer_RatingTests.m
//  Passer RatingTests
//
//  Created by Xcode User on 6/19/11.
//  Copyright 2011 Fritz Anderson. All rights reserved.
//

#import "CSVTests.h"
#import "SimpleCSVFile.h"

NSString * const	WT9TErrorDomain		= @"com.wt9t.err";
//  Replace the definition from the application delegate.

static NSString * const      kGoodGameFile = @"2010-data-calculated.csv";
static NSString * const      kTestCSVFile = @"2010-data.csv";

@implementation CSVTestBase

@synthesize csvFileName, csvFile, records, csvError, delegateBlock;

- (void)setUp
{
    [super setUp];
    
    self.records = [NSMutableArray array];
}

- (void)tearDown
{
    self.csvFileName = nil;
    self.csvFile = nil;
    self.records = nil;
    self.delegateBlock = nil;
    
    [super tearDown];
}

- (void) setCSVForFileName: (NSString *) fileName
{
//    NSString *      csvPath = [[NSBundle mainBundle]
    NSString *      csvPath = [[NSBundle bundleForClass: [self class]]
                               pathForResource: fileName
                               ofType: @""];
    STAssertNotNil(csvPath, @"Could not find %@ in main bundle", fileName);
    self.csvFile = [[[SimpleCSVFile alloc]
                     initWithPath: csvPath]
                    autorelease];
    STAssertNotNil(self.csvFile, @"file should initialize");
    self.csvFile.delegate = self;
}

- (BOOL) csvFile: (SimpleCSVFile *) file readValues: (NSDictionary *) values error: (NSError **) error
{
    STAssertNotNil(self.delegateBlock, @"Executing an abstract SimpleCSVDelegate");
    return self.delegateBlock(file, values, error);
}

@end

@interface CSVOpenTests : CSVTestBase
@end

@implementation CSVOpenTests

- (void) testNoSuchFile
{
    self.csvFile = [[[SimpleCSVFile alloc]
                     initWithPath: @"no-such-file.csv"]
                    autorelease];
    STAssertNotNil(self.csvFile, @"file should initialize");
    self.csvFile.delegate = self;

    NSError *       error;
    STAssertFalse([self.csvFile run: &error], 
                  @"nonexistent file should not run");
    STAssertNotNil(error, @"nonexistent file should return an NSError");
    STAssertEqualObjects(NSCocoaErrorDomain, [error domain],
                         @"Error domain was %@, not NSCocoaErrorDomain",
                         [error domain]);
}

- (void) testFileReadsCompletely
{
    [self setCSVForFileName: kGoodGameFile];
    self.delegateBlock = ^BOOL (SimpleCSVFile * file,
                                NSDictionary * values, 
                                NSError ** error) {
        [self.records addObject: values];
        return YES;
    };
    NSError *       error;
    BOOL            success = [self.csvFile run: &error];
    STAssertTrue(success, @"File should be valid");
    STAssertEquals(328U, self.records.count, 
                   @"%@ should return 328 records, returns %u",
                   kGoodGameFile, self.records.count);
}

- (void) testTooManyFieldsError
{
    [self setCSVForFileName: @"too-many-fields.csv"];
    self.delegateBlock = ^BOOL (SimpleCSVFile * file,
                                NSDictionary * values, 
                                NSError ** error) {
        STAssertTrue(NO, 
                     @"CSV file with the first data line bad should not call out");
        return YES;
    };
    NSError *       error;
    STAssertFalse([self.csvFile run: &error], @"File should be invalid");
    STAssertEqualObjects(WT9TErrorDomain, [error domain],
                         @"Field-count error should be in WT9TErrorDomain, was %@", 
                         [error domain]);
    STAssertNotNil(error, @"Error object after error should be non-nil.");
    //  expect = 16
    //  actual = 17
    NSDictionary *  userInfo = [error userInfo];
    STAssertEquals(16, [[userInfo objectForKey: kCSVExpectedFieldsKey] intValue],
                   @"expected fields should be 16, are %d",
                   [[userInfo objectForKey: kCSVExpectedFieldsKey] intValue]);
    STAssertEquals(17, [[userInfo objectForKey: kCSVActualFieldsKey] intValue],
                   @"expected fields should be 17, are %d",
                   [[userInfo objectForKey: kCSVActualFieldsKey] intValue]);
    STAssertEquals(2, [[userInfo objectForKey: kCSVErrorLineKey] intValue],
                   @"error line should be 2, is %d",
                   [[userInfo objectForKey: kCSVErrorLineKey] intValue]);
}


- (void) testNotEnoughFieldsError
{
    [self setCSVForFileName: @"not-enough-fields.csv"];
    self.delegateBlock = ^BOOL (SimpleCSVFile * file,
                                NSDictionary * values, 
                                NSError ** error) {
        STAssertTrue(NO, @"CSV file with the first data line bad should not call out");
        return YES;
    };
    NSError *       error;
    STAssertFalse([self.csvFile run: &error], @"File should be invalid");
    STAssertEqualObjects(WT9TErrorDomain, [error domain],
                         @"Field-count error should be in WT9TErrorDomain, was %@", 
                         [error domain]);
    STAssertNotNil(error, @"Error object after error should be non-nil.");
    //  expect = 18
    //  actual = 17
    NSDictionary *  userInfo = [error userInfo];
    STAssertEquals(18, [[userInfo objectForKey: kCSVExpectedFieldsKey] intValue],
                   @"expected fields should be 18, are %d",
                   [[userInfo objectForKey: kCSVExpectedFieldsKey] intValue]);
    STAssertEquals(17, [[userInfo objectForKey: kCSVActualFieldsKey] intValue],
                   @"expected fields should be 17, are %d",
                   [[userInfo objectForKey: kCSVActualFieldsKey] intValue]);
    STAssertEquals(2, [[userInfo objectForKey: kCSVErrorLineKey] intValue],
                   @"error line should be 2, is %d",
                   [[userInfo objectForKey: kCSVErrorLineKey] intValue]);
}

- (void) testValidCSV
{
    [self setCSVForFileName: kGoodGameFile];
    __block NSUInteger  lineCount = 0;
    self.delegateBlock = ^BOOL (SimpleCSVFile * file,
                                NSDictionary *  values,
                                NSError **      error) {
        if (lineCount == 0)
            STAssertEqualObjects(@"30", [values objectForKey: @"attempts"],
                                 @"First record mismatches attempts %@", values);
        else if (lineCount == 41)
            STAssertEqualObjects(@"Dutch", [values objectForKey: @"firstName"],
                                 @"Record 41 mismatches attempts %@", values);
       [self.records addObject: values];
        lineCount++;
        return YES;
    };
    NSError *   error;
    STAssertTrue([self.csvFile run: &error], @"Parse of %@ should succeed", kGoodGameFile);
    
    NSUInteger	recordNumber = 184;
    NSArray *	keyAndValue = [NSArray arrayWithObjects: 
                               @"firstName",@"Andy",
                               @"lastName",@"Jackson",
                               @"attempts",@"25",
                               @"completions",@"9",
                               @"interceptions",@"1",
                               @"touchdowns",@"1",
                               @"yards",@"69",
                               @"whenPlayed",@"24 Mar 2010",
                               @"ourTeam",@"Tacoma Touchdown-Scorers",
                               @"ourScore",@"19",
                               @"theirTeam",@"Glendale Centers",
                               @"theirScore",@"17",
                               @"cComp",@"0.300",
                               @"yComp",@"0.000",
                               @"tdComp",@"0.800",
                               @"intComp",@"1.375",
                               @"rating",@"41.3",
                               nil];
    NSDictionary *  record = [self.records objectAtIndex: recordNumber];
    for (NSUInteger i = 0; i < keyAndValue.count; i += 2) {
        NSString *      key = [keyAndValue objectAtIndex: i];
        NSString *      value = [keyAndValue objectAtIndex: i+1];
        STAssertEqualObjects([record objectForKey: key], value, 
                             @"Record %u key %@", recordNumber, key);
    }
    
    recordNumber = 307;
    keyAndValue = [NSArray arrayWithObjects:
                     @"firstName", @"Martin", 
                     @"lastName", @"Van Buren", 
                     @"attempts", @"5", 
                     @"completions", @"1", 
                     @"interceptions", @"1", 
                     @"touchdowns", @"1", 
                     @"yards", @"8", 
                     @"whenPlayed", @"30 Jun 2010", 
                     @"ourTeam", @"Huntington Beach Harriers", 
                     @"ourScore", @"27", 
                     @"theirTeam", @"Grand Rapids Gamers", 
                     @"theirScore", @"21", 
                     @"cComp", @"0.000", 
                     @"yComp", @"0.000", 
                     @"tdComp", @"2.375", 
                     @"intComp", @"0.000", 
                     @"rating", @"39.6",
                     nil];
    record = [self.records objectAtIndex: recordNumber];
    for (NSUInteger i = 0; i < keyAndValue.count; i += 2) {
        NSString *      key = [keyAndValue objectAtIndex: i];
        NSString *      value = [keyAndValue objectAtIndex: i+1];
        STAssertEqualObjects([record objectForKey: key], value, 
                             @"Record %u key %@", recordNumber, key);
    }
}


@end


