//
//  CoreDataTests.m
//  Passer Rating
//
//  Created by Xcode User on 7/4/11.
//  Copyright 2011 Fritz Anderson. All rights reserved.
//

#import "CoreDataTests.h"
#import "CoreDataUtilities.h"
#import "SimpleCSVFile.h"
#import "Game.h"
#import "Passer.h"

@implementation CoreDataTests

@synthesize csvFileName, csvFile, records, csvError, delegateBlock;


- (void) setCSVForFileName: (NSString *) fileName
{
    //    NSString *      csvPath = [[NSBundle mainBundle]
    NSString *      csvPath = [[NSBundle bundleForClass: [self class]]
                               pathForResource: fileName
                               ofType: @""];
    STAssertNotNil(csvPath, @"Could not find %@ in main bundle", fileName);
    self.csvFile = [[[SimpleCSVFile alloc]
                     initWithPath: csvPath]
                    autorelease];
    STAssertNotNil(self.csvFile, @"file should initialize");
    self.csvFile.delegate = self;
}

- (BOOL) csvFile: (SimpleCSVFile *) file readValues: (NSDictionary *) values error: (NSError **) error
{
    STAssertNotNil(self.delegateBlock, @"Executing an abstract SimpleCSVDelegate");
    return self.delegateBlock(file, values, error);
}

- (void) setUp
{
    [CoreDataUtilities initializeCoreData];
}

- (void) tearDown
{
    [CoreDataUtilities tearDownCoreData];
}

- (void)testSimpleRead
{
    STAssertEquals([CoreDataUtilities countObjectsForEntity: @"Game"],
                   0U, @"Should start with zero games.");
    STAssertEquals([CoreDataUtilities countObjectsForEntity: @"Passer"],
                   0U, @"Should start with zero passers.");
    
    NSString *      csvPath = [[NSBundle bundleForClass: [self class]]
                               pathForResource:  @"2010-data.csv"
                               ofType: @""];
    STAssertNotNil(csvPath, @"no path to 2010-data.csv");
    NSError *   error;
    BOOL        success = [Game loadFromCSVFile: csvPath
                                          error: &error];
    STAssertTrue(success, @"couldn't load games from 2010-data.csv");
    
    STAssertEquals([CoreDataUtilities countObjectsForEntity: @"Game"],
                   328U, @"Should load 328 games.");
    STAssertEquals([CoreDataUtilities countObjectsForEntity: @"Passer"],
                   32U, @"Should load 32 passers.");
}

@end
