//
//  CoreDataUtilities.m
//  Passer Rating
//
//  Created by Xcode User on 7/4/11.
//  Copyright 2011 Fritz Anderson. All rights reserved.
//

#import "CoreDataUtilities.h"

NSManagedObjectContext *        gMOC;

@implementation CoreDataUtilities

+ (void) initializeCoreData
{
    NSAssert1(! gMOC, @"%s: Core Data was already initialized!", __PRETTY_FUNCTION__);
    
    NSError *               error;
    NSManagedObjectModel *  mom;
    NSArray *               bundles = [NSArray arrayWithObject:
                                       [NSBundle bundleForClass: [self class]]];
    mom = [NSManagedObjectModel mergedModelFromBundles: bundles];
    NSPersistentStoreCoordinator *  psc;
    psc = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel: mom];
    NSPersistentStore *             ps;
    ps = [psc addPersistentStoreWithType: NSInMemoryStoreType
                           configuration: nil
                                     URL: nil
                                 options: nil
                                   error: &error];
    if (! ps)
        NSLog(@"%s: Can't create persistent store. %@, %@", __PRETTY_FUNCTION__, error, [error userInfo]);
    
    gMOC = [[NSManagedObjectContext alloc] init];
    [gMOC setPersistentStoreCoordinator: psc];
    [psc release];
}

+ (void) tearDownCoreData
{
    NSAssert1(gMOC, @"%s: Core Data was uninitialized!", __PRETTY_FUNCTION__);
    [gMOC release];
    gMOC = nil;
}

+ (NSUInteger) countObjectsForEntity: (NSString *) entityName
{
    NSAssert1(gMOC, @"%s requires Core Data to be initialized", __PRETTY_FUNCTION__);
    
    NSFetchRequest *    fetch = [[NSFetchRequest alloc] init];
    fetch.entity = [NSEntityDescription entityForName: entityName
                               inManagedObjectContext: gMOC];
    NSUInteger      retval = [gMOC countForFetchRequest: fetch error: NULL];
    [fetch release];
    
    return retval;
}

@end
